/* NiuTrans.Tensor - an open-source tensor library
 * Copyright (C) 2018, Natural Language Processing Lab, Northestern University.
 * All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*
* $Created by: HU Chi (huchinlp@foxmail.com) 2019-04-03
*/

#ifndef __DATASET_H__
#define __DATASET_H__

#include "../../..//tensor/XTensor.h"
#include "../../..//tensor/XGlobal.h"

#include <cstdio>
#include <fstream>
#include <unordered_map>
#include <vector>

using namespace std;
using namespace nts;

struct Example {
    int id;
    vector<int> values;
};

struct Result {
    int id;
    XTensor values;
};

using BufferType = vector<Example>;
using ResBufferType = vector<Result>;

namespace nts { // namespace nts(NiuTrans.Tensor)

/* A `DataSet` is associated with a file which contains variable length data.*/
struct DataSet {

    /* the data buffer */
    BufferType buffer;

    /* the result buffer */
    ResBufferType resBuffer;

    /* the pointer to file stream */
    ifstream* fp{nullptr};

    /* size of the data buffer */
    size_t bufferSize{ 0 };

    /* size of used data in buffer */
    size_t bufferUsed{ 0 };

    /* wether sort the dataset */
    bool sortBuffer{ true };

    /* load data from a file to the buffer */
    void LoadDataToBuffer();

    /* rerank result for output */
    void RerankRes();

    /* generate a mini-batch */
    vector<int> LoadBatch(XTensor * batchEnc, XTensor * paddingEnc, 
                   size_t batchSize, int devID);

    /* initlization function */
    void Init(const char* fname);

    /* check if the buffer is empty */
    bool IsEmpty() {
        if (bufferUsed < bufferSize)
            return false;
        return true;
    }

    /* de-constructor */
    ~DataSet() {
        if (fp)
            fp->close();
        delete fp;
    }
};

} // namespace nts(NiuTrans.Tensor)

#endif // __DATASET_H__