/* NiuTrans.Tensor - an open-source tensor library
 * Copyright (C) 2018, Natural Language Processing Lab, Northestern University.
 * All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*
 * $Created by: HU Chi (huchinlp@foxmail.com) 2019-03-18
 */

#ifndef __STRING_UTIL_H__
#define __STRING_UTIL_H__

#include <cstdlib>
#include <string>
#include <utility>
#include <vector>
using namespace std;

namespace nts {

/* Splits a string based on the given delimiter string. Each pair in the
 * returned vector has the start and past-the-end positions for each of the
 * parts of the original string. Empty fields are not represented in the output.
 */
vector<pair<int, int>> SplitToPos(const string& s, const string& delimiter);

/* Splits the given string and converts each part to the given T. */
template <typename T>
vector<T> Split(const string& s, const string& delimiter);

template <>
inline vector<string> Split(const string& s, const string& delimiter)
{
    vector<string> fields;
    for (const auto& p : SplitToPos(s, delimiter)) {
        fields.emplace_back(s.substr(p.first, p.second - p.first));
    }
    return fields;
}

template <>
inline vector<int> Split(const string& s, const string& delimiter)
{
    vector<int> fields;
    for (const auto& p : SplitToPos(s, delimiter)) {
        fields.emplace_back(strtol(s.data() + p.first, nullptr, 10));
    }
    return fields;
}

template <>
inline vector<int64_t> Split(const string& s, const string& delimiter)
{
    vector<int64_t> fields;
    for (const auto& p : SplitToPos(s, delimiter)) {
        fields.emplace_back(strtoll(s.data() + p.first, nullptr, 10));
    }
    return fields;
}

template <>
inline vector<float> Split(const string& s, const string& delimiter)
{
    vector<float> fields;
    for (const auto& p : SplitToPos(s, delimiter)) {
        fields.emplace_back(strtof(s.data() + p.first, nullptr));
    }
    return fields;
}

template <>
inline vector<uint8_t> Split(const string& s, const string& delimiter)
{
    vector<uint8_t> fields;
    for (const auto& p : SplitToPos(s, delimiter)) {
        fields.emplace_back(strtol(s.data() + p.first, nullptr, 10));
    }
    return fields;
}

template <>
inline vector<bool> Split(const string& s, const string& delimiter)
{
    vector<bool> fields;
    for (const auto& p : SplitToPos(s, delimiter)) {
        fields.emplace_back(
            static_cast<bool>(strtol(s.data() + p.first, nullptr, 10)));
    }
    return fields;
}

} // namespace nts

#endif // __STRING_UTIL_H__
